/* Pilot serial I/O test program
 * Rev 1 : Add chart record function
 * Ingeneering Inc.
 * www.ingeninc.com
 * jkh, 11/1/97
 * jkh@acm.org
 */


/* includes */
#include <Pilot.h>			// system
#include <SerialMgr.h>
#include "HiLo_res.h"		// app


/* defines */
#define	ChartRectLeft		30
#define	ChartRectTop		52
#define	ChartRectWidth		100
#define	ChartRectHeight		90
#define MaxDataPoints		ChartRectWidth
#define VertScale			ChartRectHeight / 100


/* global variables */
static	UInt				SerIOAvg = 0;
static	UInt				SerIOCnt = 0;
static	UInt				SerIOCur = 0;
static	UInt				SerIOMax = 0;
static	UInt				SerIOMin = 257;
static	UInt				SerIORef = 0;
static	UInt				SerIOSum = 0;
static	SerSettingsType		SerIOSettings;
static 	Boolean				SerIOConnected = false;
static	RectangleType		ChartRecRect;
static	Short				Data[ChartRectWidth];
static	Short				DataPoints = 0;


/* messages */
char	*strErrSerSetup = "Error setting up serial port";
char	*strErrSerPortOpen = "Error opening serial port";
char	*strErrRcvChk = "Serial receive check error";
char	*strErrRcvErr = "Serial receive error";
char	*strOKSerClose = "Serial port close OK";
char	*strErrSerClose = "Serial port close error: ";
char	*strErrSerLibOpen = "Error opening serial library";
char	*StrBlank = " ";
		

/* Function prototypes */
static void StartApplication(void);
static void StopApplicatoin(void);
static Boolean MainFormHandleEvent(EventPtr event);
static void EventLoop(void);

static void SerIOOpen(void);
static void SerIOClose(void);
static void SerIOReceive(void);

static void ShowNumResultFld (Word FieldID, ULong val);
static void EnableControl (FormPtr Frm, Word ControlID, Boolean State);
static void EnableField (FormPtr Frm, Word FieldID, Boolean State);

static void CreateChartRec(void);
static void AddPoint (Short pt);


/*
 * SerIOReceive
 * R:\CodeWarrior\PalmPilot WinSDK\Apps\Working Folder\PalmPilotApp.prc
 */
static void SerIOReceive(void) {
	ULong		RcvCount = 0;
	Err 		error;
	Byte		BufIn[17];
	
	if (!SerIORef || !SerIOConnected)
		return;
	
	error = SerReceiveCheck(SerIORef, &RcvCount);
	if (error) {	
		SerClearErr(SerIORef);
		FrmCustomAlert (AlertGenericAlert, strErrRcvChk, StrBlank, StrBlank);
		return;
	}
	
	if ((RcvCount <= 0))
		return;
		
	if (RcvCount > 1)
		RcvCount = 1;

	// Palm OS1, OS2 compatible
	error = SerReceive10(SerIORef, BufIn, 1, 10);
	
	// Palrm OS2 compatible only
	//error = 0;
	//RcvCount = SerReceive(SerIORef, BufIn, 1, 0, &error);
	
	if (error) {
		SerClearErr(SerIORef);
		FrmCustomAlert (AlertGenericAlert, strErrRcvErr, StrBlank, StrBlank);
		return;
	}

	SerIOCnt++;
	SerIOCur = BufIn[0];
	SerIOSum += SerIOCur;
	if (SerIOCnt > 0)
		SerIOAvg = SerIOSum / SerIOCnt;
	if (SerIOCur < SerIOMin)
		SerIOMin = SerIOCur;
	if (SerIOCur > SerIOMax)
		SerIOMax = SerIOCur;

	// display results
	ShowNumResultFld (HiLoMainFldAvgField, SerIOAvg);
	ShowNumResultFld (HiLoMainFldRxCountField, SerIOCnt);
	ShowNumResultFld (HiLoMainFldLoField, SerIOMin);
	ShowNumResultFld (HiLoMainFldHiField, SerIOMax);
	ShowNumResultFld (HiLoMainFldLastField, SerIOCur);
	
	AddPoint (SerIOCur);
	
}


/*
 * ShowNumResultFld
 */
static void ShowNumResultFld (Word FieldID, ULong val) {
	FormPtr 	Frm;
	FieldPtr 	FldPtr;
	char		Buf[32];
	
	Frm = FrmGetFormPtr(HiLoMainForm);
	FldPtr = (FieldPtr)(FrmGetObjectPtr(Frm, (FrmGetObjectIndex(Frm, FieldID))));
	StrIToA(Buf, val);
	FldDelete(FldPtr, 0, FldGetTextLength(FldPtr));
	FldInsert(FldPtr, Buf, StrLen(Buf));
}


/*
 * AddPoint
 */
static void AddPoint (Short pt) {
	Short		LineHt;

	if (DataPoints < MaxDataPoints) {
		Data[DataPoints] = pt;
		LineHt = (int) (pt * VertScale + 1);
		LineHt = (LineHt > ChartRectHeight) ? ChartRectHeight : LineHt;
		
		ShowNumResultFld (HiLoMainFldLastField, LineHt);		
		
		WinDrawGrayLine(ChartRectLeft + DataPoints,
						ChartRectTop + ChartRectHeight - 1,
						ChartRectLeft + DataPoints,
						ChartRectTop + ChartRectHeight - LineHt);
		DataPoints++;
	}
}


/*
 * EnableControl
 */
static void EnableControl (FormPtr Frm, Word ControlID, Boolean State) {
	ControlPtr 	cptr;
	
	cptr = (ControlPtr)(FrmGetObjectPtr(Frm, (FrmGetObjectIndex(Frm, ControlID))));
	if (State)
		CtlShowControl(cptr);
	else
		CtlHideControl(cptr);
}


/*
 * EnableField
 */
static void EnableField (FormPtr Frm, Word FieldID, Boolean State) {
	FieldPtr	fptr;
	fptr = (FieldPtr)(FrmGetObjectPtr(Frm, (FrmGetObjectIndex(Frm,FieldID))));
	if (State)
		FldDrawField(fptr);
	else
		FldEraseField(fptr);
}


/*
 * SerIOOpen
 */
static void SerIOOpen(void) {
	Err		Error;
	FormPtr 	Frm;
	FieldPtr	fptr;
	
	Error = SerOpen(SerIORef, 0, 9600);
	if (Error) {
		FrmCustomAlert (AlertGenericAlert, strErrSerPortOpen, StrBlank, StrBlank);
		SerClearErr(SerIORef);
		return;
	}
	
	SerIOSettings.baudRate = 9600;
	SerIOSettings.ctsTimeout = serDefaultCTSTimeout;
	SerIOSettings.flags =  	serSettingsFlagStopBits1 |
							serSettingsFlagBitsPerChar8	|
							serSettingsFlagCTSAutoM |
							serSettingsFlagRTSAutoM;
	
	Error = SerSetSettings(SerIORef, &SerIOSettings);
	if (Error) {
		FrmCustomAlert (AlertGenericAlert, strErrSerSetup, StrBlank, StrBlank);
		SerClearErr(SerIORef);
		return;		
		}
	
	Frm = FrmGetFormPtr(HiLoMainForm);

	EnableControl(Frm, HiLoMainBtnExitButton, true);
	EnableControl(Frm, HiLoMainBtnGoButton, false);

	EnableField(Frm, HiLoMainFldLoField, true);
	EnableField(Frm, HiLoMainFldHiField, true);
	EnableField(Frm, HiLoMainFldAvgField, true);
	EnableField(Frm, HiLoMainFldRxCountField, true);
	EnableField(Frm, HiLoMainFldLastField, true);
	
	fptr = (FieldPtr)(FrmGetObjectPtr(Frm, (FrmGetObjectIndex(Frm,HiLoMainFldLastField))));
	FldGrabFocus(fptr);

	SerIOConnected = true;
}


/*
 * SerIOClose
 */
static void SerIOClose(void) {
	char		BufSmall[17];
	char		BufLarge[80];
	Err			Error;
	
	if (!SerIOConnected)
		return;
		
	Error = SerClose (SerIORef);
	if (Error) {
		StrIToA(BufSmall, Error);
		StrCopy(BufLarge, strErrSerClose);
		StrCat(BufLarge, BufSmall);
		FrmCustomAlert (AlertGenericAlert, BufLarge, StrBlank, StrBlank);
	}
}


/*
 * CreateChartRec
 */
static void CreateChartRecRect(void) {
	
	ChartRecRect.topLeft.x = ChartRectLeft;
	ChartRecRect.topLeft.y = ChartRectTop;
	ChartRecRect.extent.x = ChartRectWidth;
	ChartRecRect.extent.y = ChartRectHeight;
	
	WinDrawRectangleFrame(simpleFrame, &ChartRecRect);
}


/*
 * StopApplication
 */
static void StopApplication(void) {
	SerIOClose();
}


/*
 * MainFormHandleEvent
 */
static Boolean MainFormHandleEvent(EventPtr event) {
	Boolean		handled = false;
	EventType	newEvent;
	
	if (event->eType == ctlSelectEvent) {
		switch (event->data.ctlEnter.controlID) {
			case HiLoMainBtnExitButton:
			   	MemSet(&newEvent, sizeof(EventType), 0);
			   	newEvent.eType = appStopEvent;
			   	EvtAddEventToQueue(&newEvent);
   				break;

   			case HiLoMainBtnGoButton:
				SerIOOpen();
   				break;

   		}
		handled = true;
	}

	else if (event->eType == nilEvent) {
		SerIOReceive ();
		handled = true;
	}
		
	return handled;
}


/*
 * EventLoop
 */
static void EventLoop(void) {
	EventType	event;
	//Word		error;
	
	do {
		EvtGetEvent(&event, evtWaitForever);
		if (! SysHandleEvent(&event))
			//if (! MenuHandleEvent(CurrentMenu, &event, &error))
				if (! MainFormHandleEvent(&event))
					FrmHandleEvent(FrmGetActiveForm(), &event);
	}
	while (event.eType != appStopEvent);
}


/*
 * StartApplication
 */
static void StartApplication(void) {
	Err			Error;
	FormPtr		Frm;

	// Initialize and draw the main form.
	Frm = FrmInitForm(HiLoMainForm);	
	FrmSetActiveForm(Frm);
	FrmDrawForm(Frm);

	EnableControl(Frm, HiLoMainBtnExitButton, true);
	EnableField(Frm, HiLoMainFldLoField, false);
	EnableField(Frm, HiLoMainFldHiField, false);
	EnableField(Frm, HiLoMainFldAvgField, false);
	EnableField(Frm, HiLoMainFldRxCountField, false);
	EnableField(Frm, HiLoMainFldLastField, false);

	Error = SysLibFind("Serial Library", &SerIORef);
	if (Error) {
		FrmCustomAlert (AlertGenericAlert, strErrSerLibOpen, StrBlank, StrBlank);
		SerClearErr(SerIORef);
		return;		
	}
	
	CreateChartRecRect();
}


/*
 * PilotMain
 */
DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags) {
	if (cmd == sysAppLaunchCmdNormalLaunch)	{
		StartApplication();
		EventLoop();
		StopApplication();
	}
	return 0;
}

